// 3DGraphFrame.cpp: implementation of the C3DGraphFrame class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "3DGraphFrame.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

C3DGraphFrame::C3DGraphFrame():
	m_p3DGraph(NULL),
	m_flAngleXY(0), 
	m_flAngleZ(45), 
	m_flRadius(50),
	m_flAngleStep(2), 
	m_flRadiusStep(0.5)
{

}

C3DGraphFrame::~C3DGraphFrame()
{

}

HRESULT C3DGraphFrame::Create3DFrame(HWND p_hwndWindow, C3DGraphic* p_p3DGraph)
{
	BOOL br = FALSE;
	HRESULT hr = E_FAIL;

	ATLASSERT(p_hwndWindow);
	if (!p_hwndWindow) {
		return E_INVALIDARG;
	}
	br = CWindowImpl<C3DGraphFrame>::SubclassWindow(p_hwndWindow);
	if (!br) {
		return E_FAIL;
	}
	
	m_p3DGraph = p_p3DGraph;
	hr = Init3DFrame();
	ATLASSERT(SUCCEEDED(hr));
	if (FAILED(hr)) {
		return hr;
	}

	return S_OK;
}

HRESULT C3DGraphFrame::UpdateEyePosition(BOOL p_b_ReDraw)
{
	if (!m_p3DGraph) {
		return E_UNEXPECTED;
	}

	FLOAT flAngleXYRad = D3DXToRadian(m_flAngleXY);
	FLOAT flAngleZRad = D3DXToRadian(m_flAngleZ);
	
	FLOAT flX = m_flRadius * cos(flAngleZRad) * cos(flAngleXYRad);	
	FLOAT flY = m_flRadius * cos(flAngleZRad) * sin(flAngleXYRad);	
	FLOAT flZ = m_flRadius * sin(flAngleZRad);

	return m_p3DGraph->SetEyeAt(flX, flY, flZ, p_b_ReDraw);
}

HRESULT C3DGraphFrame::Init3DFrame()
{
	return UpdateEyePosition(FALSE);
}

HRESULT C3DGraphFrame::Refresh(BOOL p_bRecalcData)
{
	HRESULT hr = m_p3DGraph->ReRender(p_bRecalcData);
	InvalidateRect(NULL);
	return hr;
}
