// 3DGraphFrame.h: interface for the C3DGraphFrame class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_3DGRAPHFRAME_H__94F0886B_7CEA_43C2_916E_589933B2BABB__INCLUDED_)
#define AFX_3DGRAPHFRAME_H__94F0886B_7CEA_43C2_916E_589933B2BABB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <atlwin.h>
#include "3DGraphic.h"

class C3DGraphFrame: public CWindowImpl<C3DGraphFrame>
{
public:	
/////////////////////////////////////////////////////////////////////////////////////////
//  C3DGraphFrame construction/destruction
//
	C3DGraphFrame();
	virtual ~C3DGraphFrame();

/////////////////////////////////////////////////////////////////////////////////////////
//  C3DGraphFrame message map
//
private:
	BEGIN_MSG_MAP(CBoardCtrl)
		MESSAGE_HANDLER(WM_PAINT, OnPaint)
		MESSAGE_HANDLER(WM_GETDLGCODE, OnGetDlgCode)
		MESSAGE_HANDLER(WM_KEYDOWN, OnKeyDown)
		MESSAGE_HANDLER(WM_CHAR, OnChar)
		bHandled = FALSE;
	END_MSG_MAP()
		
/////////////////////////////////////////////////////////////////////////////////////////
//  C3DGraphFrame messages handlers
//
private:
	LRESULT OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		if (m_p3DGraph) {
			m_p3DGraph->BlitToWindow();
		}
		ValidateRect(NULL);
		return 0;
	}

	LRESULT OnGetDlgCode(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		return DLGC_WANTALLKEYS;
	}

	LRESULT OnKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		if (!m_p3DGraph) {
			return 0;
		}
		switch(wParam) 
		{
		case(VK_DOWN):
			{
				m_flAngleZ -= m_flAngleStep;
				UpdateEyePosition();
				break;
			}
		case(VK_UP):
			{
				m_flAngleZ += m_flAngleStep;
				UpdateEyePosition();
				break;
			}
		case(VK_LEFT):
			{
				m_flAngleXY -= m_flAngleStep;
				UpdateEyePosition();
				break;
			}
		
		case(VK_RIGHT):
			{
				m_flAngleXY += m_flAngleStep;
				UpdateEyePosition();
				break;
			}
			
		}
		return 0;
	}

	LRESULT OnChar(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		if (!m_p3DGraph) {
			return 0;
		}
		switch(wParam) 
		{
		case('+'):
			{
				m_flRadius -= m_flRadiusStep;
				UpdateEyePosition();
				break;
			}
		case('-'):
			{
				m_flRadius += m_flRadiusStep;
				UpdateEyePosition();
				break;
			}
		}
		return 0;
	}

/////////////////////////////////////////////////////////////////////////////////////////
//  C3DGraphFrame methods
//
private:
	
protected:
	HRESULT Init3DFrame();	
public:
	HRESULT Create3DFrame(HWND p_hwndWindow, C3DGraphic* p_p3DGraph);
	HRESULT Refresh(BOOL p_bRecalcData = FALSE);

	HRESULT UpdateEyePosition(BOOL p_b_ReDraw = TRUE);
/////////////////////////////////////////////////////////////////////////////////////////
// C3DGraphFrame member variables 
//
private:
	
protected:
	C3DGraphic* m_p3DGraph;

	// Current eye position. Angles in degrees.
	// 
	FLOAT m_flAngleXY, m_flAngleZ, m_flRadius;
	FLOAT m_flAngleStep, m_flRadiusStep;
public:
	
/////////////////////////////////////////////////////////////////////////////////////////
// C3DGraphFrame static methods  
//
private:
	
protected:
	
public:
	
/////////////////////////////////////////////////////////////////////////////////////////
// C3DGraphFrame static member variables 
//
private:
	
protected:
	
public:
	
};


#endif // !defined(AFX_3DGRAPHFRAME_H__94F0886B_7CEA_43C2_916E_589933B2BABB__INCLUDED_)
