// 3DGraphic.h: interface for the C3DGraphic class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_3DGRAPHIC_H__22338364_0C05_4230_A4D7_01BF34E62478__INCLUDED_)
#define AFX_3DGRAPHIC_H__22338364_0C05_4230_A4D7_01BF34E62478__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <stdafx.h>
#include "D3D8App.h"

#ifndef MIN_XRANGE_DEFAULT
#define MIN_XRANGE_DEFAULT -10.0f
#endif

#ifndef MAX_XRANGE_DEFAULT
#define MAX_XRANGE_DEFAULT +10.0f
#endif

#ifndef MIN_YRANGE_DEFAULT
#define MIN_YRANGE_DEFAULT -10.0f
#endif

#ifndef MAX_YRANGE_DEFAULT
#define MAX_YRANGE_DEFAULT +10.0f
#endif

#define MAX_VERTEXES 2000

typedef struct {
    FLOAT x, y, z;
	FLOAT nx, ny, nz;
} GRAPH3DVERTEXSTRUCT;

#define D3DFVF_GRAPH3DVERTEX (D3DFVF_XYZ | D3DFVF_NORMAL)



class C3DFunction
{
/////////////////////////////////////////////////////////////////////////////////////////
// C3DFunction construction/destruction
//
public:
	C3DFunction() {}

/////////////////////////////////////////////////////////////////////////////////////////
// C3DFunction interface
//
public:	
	virtual ~C3DFunction() {}
	
//	methods for retrieving default graphic area.
//  If function doesn't support default area it must return S_FALSE.
	virtual HRESULT GetXRange(FLOAT* p_pflXMin, FLOAT* p_pflXMax) = 0;
	virtual HRESULT GetYRange(FLOAT* p_pflYMin, FLOAT* p_pflYMax) = 0;

//  Method for calculating function value in given point.
	virtual HRESULT GetZValue(FLOAT p_flX, FLOAT p_flY, FLOAT* p_pflZ) = 0;
};

/////////////////////////////////////////////////////////////////////////////////////////
// CGraphGrid - helper class for discretize function;
//
class CGraphGrid
{
/////////////////////////////////////////////////////////////////////////////////////////
// C3DFunction construction/destruction
//
public:
	CGraphGrid(C3DFunction* p_pFunction, 
		FLOAT p_flXMin, FLOAT p_flXMax,
		FLOAT p_flYMin, FLOAT p_flYMax):

		m_flXMin(p_flXMin),
		m_flXMax(p_flXMax),
		m_flYMin(p_flYMin),
		m_flYMax(p_flYMax),
		m_pFunction(p_pFunction),
		m_dwPointsPerX(0),
		m_pflXAxe(NULL),
		m_dwPointsPerY(0),
		m_pflYAxe(NULL),
		m_pflGridData(NULL)
			
	{
		
	}

	~CGraphGrid()
	{
		ClearGrid();
	}
/////////////////////////////////////////////////////////////////////////////////////////
// CGraphGrid member variables 
//
public:

protected:
	FLOAT m_flXMin, m_flXMax, m_flYMin, m_flYMax;
	C3DFunction* m_pFunction;

	DWORD  m_dwPointsPerX;
	FLOAT* m_pflXAxe;

	DWORD  m_dwPointsPerY;
	FLOAT* m_pflYAxe;

	FLOAT* m_pflGridData;
	
private:

/////////////////////////////////////////////////////////////////////////////////////////
// CGraphGrid methods 
//
public:
	HRESULT CalcGridFromMaxPoints(DWORD p_dwMaxPoints)
	{
		if (4 > p_dwMaxPoints) {
			// We cannot have less than 2 points per axe.
			return E_INVALIDARG;
		}

		DWORD dwPointsPerAxe = (DWORD)floor(sqrt((FLOAT)p_dwMaxPoints));
		return CalcGridHelper(dwPointsPerAxe, dwPointsPerAxe);
	}
	HRESULT CalcGridFromNumPoints(DWORD p_dwPointsPerX, DWORD p_dwPointsPerY)
	{
		return E_NOTIMPL; //yet...
	}
	
	DWORD GetPointsPerX() {return m_dwPointsPerX;}
	DWORD GetPointsPerY() {return m_dwPointsPerY;}
	
	FLOAT* GetXAxeData() {return m_pflXAxe;}
	FLOAT* GetYAxeData() {return m_pflYAxe;}
	
	// Data are stored in this way: X1Y1, X2Y1, X3Y1, X1Y2, X2Y2.... and so on.
	FLOAT* GetGridData() {return m_pflGridData;}
	
protected:
	HRESULT CalcGridHelper(DWORD p_dwPointsPerX, DWORD p_dwPointsPerY)
	{
		if (!ClearGrid()) {
			return E_FAIL;
		}
		m_dwPointsPerX = p_dwPointsPerX;
		m_dwPointsPerY = p_dwPointsPerY;
		
		m_pflXAxe = new FLOAT[m_dwPointsPerX];
		if (!m_pflXAxe) {
			ClearGrid();
			return E_OUTOFMEMORY;
		}

		m_pflYAxe = new FLOAT[m_dwPointsPerY];
		if (!m_pflYAxe) {
			ClearGrid();
			return E_OUTOFMEMORY;
		}
		m_pflGridData = new FLOAT[m_dwPointsPerX * m_dwPointsPerY];
		if (!m_pflGridData) {
			ClearGrid();
			return E_OUTOFMEMORY;
		}
		for (INT i = 0; i < m_dwPointsPerX; i++) {
			m_pflXAxe[i] = m_flXMin + (m_flXMax - m_flXMin) * i / (m_dwPointsPerX - 1);
		}

		for (INT j = 0; j < m_dwPointsPerY; j++) {
			m_pflYAxe[j] = m_flYMin + (m_flYMax - m_flYMin) * j / (m_dwPointsPerY - 1);
		}
		
		FLOAT flValue;
		HRESULT hr;
		for (j = 0; j < m_dwPointsPerY; j++) {
			for (i = 0; i < m_dwPointsPerX; i++) {
				hr = m_pFunction->GetZValue(m_pflXAxe[i], m_pflYAxe[j], &flValue);
				// We really do not need do anything if an error occured.
				// Author of C3DFunction might take a care of it.
				m_pflGridData[i + m_dwPointsPerX * j] = flValue;
			}
		}

		return S_OK;
		
	}

	BOOL ClearGrid()
	{
		if (m_pflXAxe) {
			delete[] m_pflXAxe;
			m_pflXAxe = NULL;
		}
		if (m_pflYAxe) {
			delete[] m_pflYAxe;
			m_pflYAxe = NULL;
		}
		if (m_pflGridData) {
			delete[] m_pflGridData;
			m_pflGridData = NULL;
		}
		return TRUE;
	}
private:
	
};




class C3DGraphic 
{
public:	
/////////////////////////////////////////////////////////////////////////////////////////
//  C3DGraphic construction/destruction
//
	C3DGraphic();
	virtual ~C3DGraphic();
	
/////////////////////////////////////////////////////////////////////////////////////////
//  C3DGraphic methods
//
private:

protected:
// Internal methods-helpers
	// Allocacting memory for vertexes
	HRESULT CreateVertexBuffers(DWORD p_dwVertexesNum);	
	// Recalculate data from our function. Attention!!! Can be VERY time consuming!!!
	HRESULT RecalculateData();

	// Calculate normal to vertex in given point of grid(p_dwX, p_dwY)
	HRESULT CalcNormal(CGraphGrid* p_pGrid, DWORD p_dwX, DWORD p_dwY, 
		GRAPH3DVERTEXSTRUCT* p_pVertex);

// Set rendering ambient.
	// Set light parameters: position, type, color etc.
	HRESULT InitLight();
	// Set material parameters.
	HRESULT InitMaterial();
		
public:
// Creation and destroying methods
	HRESULT Create(CD3D8Application* p_p3DApplication, 
		HWND p_hwndTarget, C3DFunction* p_pFunction);

// Vertex amount methods	
	HRESULT SetMaxVertexesPerGraph(DWORD p_dwMaxVertexes);
	DWORD GetMaxVertexPerGraph() {return m_dwMaxVertexes;}
	
	
// Target math function methods	
	HRESULT SetFunction(C3DFunction* p_pFunction);
	C3DFunction* GetFunction() {return m_pFunction;}



// Function area methods
	HRESULT SetXMin(FLOAT p_flXMin);
	HRESULT SetXMax(FLOAT p_flXMax);
	HRESULT SetYMin(FLOAT p_flYMin);
	HRESULT SetYMax(FLOAT p_flYMax);

	FLOAT GetXMin() {return m_flXMin;}
	FLOAT GetXMax() {return m_flXMax;}
	FLOAT GetYMin() {return m_flYMin;}
	FLOAT GetYMax() {return m_flYMax;}

// Eye position methods
	HRESULT SetEyeAt(FLOAT p_flX, FLOAT p_flY, FLOAT p_flZ, BOOL p_bRedraw = TRUE);
	HRESULT SetEyeTo(FLOAT p_flX, FLOAT p_flY, FLOAT p_flZ, BOOL p_bRedraw = TRUE);
	HRESULT GetEyeAt(PFLOAT p_pflX, PFLOAT p_pflY, PFLOAT p_pflZ);
	HRESULT GetEyeTo(PFLOAT p_pflX, PFLOAT p_pflY, PFLOAT p_pflZ);
	
// Rendering methods
	
	// Render data to back buffer
	HRESULT ReRender(BOOL p_bRecalcData = FALSE);
	// Blit data from back buffer to primary surface i.e. to screen
	HRESULT BlitToWindow();

// Set rendering ambient parameters.
	HRESULT SetBackColor(FLOAT p_flR, FLOAT p_flG, FLOAT p_flB);
	HRESULT GetBackColor(PFLOAT p_pflR, PFLOAT p_pflG, PFLOAT p_pflB);

	HRESULT SetLightDirection(FLOAT p_flX, FLOAT p_flY, FLOAT p_flZ);
	HRESULT GetLightDirection(PFLOAT p_pflX, PFLOAT p_pflY, PFLOAT p_pflZ);

	HRESULT SetLightAmbient(FLOAT p_flR, FLOAT p_flG, FLOAT p_flB);
	HRESULT SetLightDiffuse(FLOAT p_flR, FLOAT p_flG, FLOAT p_flB);
	HRESULT SetLightSpecular(FLOAT p_flR, FLOAT p_flG, FLOAT p_flB);
	
	HRESULT SetMaterialAmbient(FLOAT p_flR, FLOAT p_flG, FLOAT p_flB);
	HRESULT SetMaterialDiffuse(FLOAT p_flR, FLOAT p_flG, FLOAT p_flB);
	HRESULT SetMaterialEmissive(FLOAT p_flR, FLOAT p_flG, FLOAT p_flB);
	HRESULT SetMaterialSpecular(FLOAT p_flR, FLOAT p_flG, 
		FLOAT p_flB, FLOAT p_flP);


/////////////////////////////////////////////////////////////////////////////////////////
// C3DGraphic member variables 
//
private:

protected:
	FLOAT m_flXMin, m_flXMax, m_flYMin, m_flYMax;
	HWND m_hwndRenderTarget;
	C3DFunction* m_pFunction;

	IDirect3DDevice8* m_p3DDevice;

	CD3D8Application* m_p3DApplication;

	DWORD m_dwMaxVertexes;

	DWORD m_dwElementsInVB;
	IDirect3DVertexBuffer8* m_pDataVB;
	
	D3DXVECTOR3 m_vecEyeAt;
	D3DXVECTOR3 m_vecEyeTo;

// Ambient parameters: material, light etc...	
	D3DMATERIAL8	m_theGraphMaterial;
	D3DLIGHT8		m_theGraphLight;
	D3DCOLOR		m_dwBackColor;
public:

/////////////////////////////////////////////////////////////////////////////////////////
// C3DGraphic static methods  
//
private:

protected:

public:

/////////////////////////////////////////////////////////////////////////////////////////
// C3DGraphic static member variables 
//
private:

protected:

public:

};


#endif // !defined(AFX_3DGRAPHIC_H__22338364_0C05_4230_A4D7_01BF34E62478__INCLUDED_)
