
#include "StdAfx.h"
#include "MainDlg.h"

CMainDlg::CMainDlg()
{

}

HRESULT CMainDlg::CreatePropertiesWindows()
{
	// Set notifees
	
	BOOL br= FALSE;
	br = m_theMaterialPropsWindow.SetNotifee(this);
	if (!br) {
		return E_FAIL;
	}

	br = m_theLightPropsWindow.SetNotifee(this);
	if (!br) {
		return E_FAIL;
	}

	br = m_theBackColorWindow.SetNotifee(this);
	if (!br) {
		return E_FAIL;
	}

	br = m_theFunctionTypeWindow.SetNotifee(this);
	if (!br) {
		return E_FAIL;
	}

	

	HWND hWnd = NULL;
	hWnd = m_theLightPropsWindow.Create(NULL);
	ATLASSERT(hWnd);
	if (!hWnd) {
		return E_FAIL;
	}
	
	hWnd = m_theMaterialPropsWindow.Create(NULL);
	ATLASSERT(hWnd);
	if (!hWnd) {
		return E_FAIL;
	}

	hWnd = m_theBackColorWindow.Create(NULL);
	ATLASSERT(hWnd);
	if (!hWnd) {
		return E_FAIL;
	}

	hWnd = m_theFunctionTypeWindow.Create(NULL);
	ATLASSERT(hWnd);
	if (!hWnd) {
		return E_FAIL;
	}

	return S_OK;

}

HRESULT CMainDlg::HideRequest(UINT p_uiPropWindowID)
{
	return ShowPropertiesWindow(p_uiPropWindowID, FALSE);
}

HRESULT CMainDlg::ParametersChanged(UINT p_uiPropWindowID)
{
	HRESULT hr;hr;
	if (CMaterialPropsWindow::IDD == p_uiPropWindowID) {
		return MaterialParametersChanged();
	}
	if (CLightPropsWindow::IDD == p_uiPropWindowID) {
		return LightParametersChanged();
	}
	if (CBackColorWindow::IDD == p_uiPropWindowID) {
		return BackColorChanged();
	}
	if (CFunctionTypeWindow::IDD == p_uiPropWindowID) {
		return FunctionTypeChanged();
	}

	
	return S_OK;
}

HRESULT CMainDlg::ShowPropertiesWindow(UINT p_uiPropWindowID, BOOL p_bShow)
{
	UINT uiShowCommand;
	UINT uiMenuCommand;
	if (p_bShow) {
		uiShowCommand = SW_SHOW;
		uiMenuCommand = MF_CHECKED;
	} else {
		uiShowCommand = SW_HIDE;
		uiMenuCommand = MF_UNCHECKED;
	}
	switch(p_uiPropWindowID)
	{
	case(CLightPropsWindow::IDD):
		{
			m_theLightPropsWindow.ShowWindow(uiShowCommand);
			CheckMenuItem(GetMenu(), IDC_LIGHTPROPERTIES, 
				MF_BYCOMMAND | uiMenuCommand);
			
			break;
		}
	case(CMaterialPropsWindow::IDD):
		{
			m_theMaterialPropsWindow.ShowWindow(uiShowCommand);
			CheckMenuItem(GetMenu(), IDC_MATERIALPROPERTIES, 
				MF_BYCOMMAND | uiMenuCommand);
			
			break;
		}
	case(CBackColorWindow::IDD):
		{
			m_theBackColorWindow.ShowWindow(uiShowCommand);
			CheckMenuItem(GetMenu(), IDC_BACKGROUNDCOLOR, 
				MF_BYCOMMAND | uiMenuCommand);
			
			break;
		}
	case(CFunctionTypeWindow::IDD):
		{
			m_theFunctionTypeWindow.ShowWindow(uiShowCommand);
			CheckMenuItem(GetMenu(), IDC_FUNCTIONTYPE, 
				MF_BYCOMMAND | uiMenuCommand);
			
			break;
		}

	default:
		{
			return E_INVALIDARG;
		}

	}
	return S_OK;
}

HRESULT CMainDlg::MaterialParametersChanged()
{
	FLOAT flR, flG, flB, flP; 
	HRESULT hr;

	hr = m_theMaterialPropsWindow.GetAmbientComponent(&flR, &flG, &flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetMaterialAmbient(flR, flG, flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_theMaterialPropsWindow.GetDiffuseComponent(&flR, &flG, &flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetMaterialDiffuse(flR, flG, flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_theMaterialPropsWindow.GetEmissiveComponent(&flR, &flG, &flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetMaterialEmissive(flR, flG, flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_theMaterialPropsWindow.GetSpecularComponent(&flR, &flG, 
		&flB, &flP);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetMaterialSpecular(flR, flG, flB, flP);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_theGraphFrame.Refresh();
	ATLASSERT(SUCCEEDED(hr));
	return S_OK;
}

HRESULT CMainDlg::LightParametersChanged()
{
	FLOAT flR, flG, flB;
	FLOAT flX, flY, flZ;
	
	HRESULT hr;

	hr = m_theLightPropsWindow.GetAmbientComponent(&flR, &flG, &flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetLightAmbient(flR, flG, flB);
	ATLASSERT(SUCCEEDED(hr));

	
	hr = m_theLightPropsWindow.GetDiffuseComponent(&flR, &flG, &flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetLightDiffuse(flR, flG, flB);
	ATLASSERT(SUCCEEDED(hr));

	
	hr = m_theLightPropsWindow.GetSpecularComponent(&flR, &flG, &flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetLightSpecular(flR, flG, flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_theLightPropsWindow.GetLightDirection(&flX, &flY, &flZ);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetLightDirection(flX, flY, flZ);
	ATLASSERT(SUCCEEDED(hr));


	hr = m_theGraphFrame.Refresh();
	ATLASSERT(SUCCEEDED(hr));
	return S_OK;
}

HRESULT CMainDlg::BackColorChanged()
{
	FLOAT flR, flG, flB; 
	HRESULT hr = E_FAIL;

	hr = m_theBackColorWindow.GetBackColor(&flR, &flG, &flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_the3DGraphic.SetBackColor(flR, flG, flB);
	ATLASSERT(SUCCEEDED(hr));

	hr = m_theGraphFrame.Refresh();
	ATLASSERT(SUCCEEDED(hr));

	return S_OK;
}

HRESULT CMainDlg::FunctionTypeChanged()
{
	HRESULT hr;

	INT nCurrFunction = m_theFunctionTypeWindow.GetCurrentFunctionID();
	switch(nCurrFunction) 
	{
	case(IDC_RB_FUNCTIONTYPE_SINX):
		{
			hr = m_the3DGraphic.SetFunction(&m_theSplashFunction);
			ATLASSERT(SUCCEEDED(hr));
			if (FAILED(hr)) {
				return hr;
			}
			break;
		}
	case(IDC_RB_FUNCTIONTYPE_PLANE):
		{
			hr = m_the3DGraphic.SetFunction(&m_thePlaneFunction);
			ATLASSERT(SUCCEEDED(hr));
			if (FAILED(hr)) {
				return hr;
			}
			break;
		}
	case(IDC_RB_FUNCTIONTYPE_PARABOLOID):
		{
			hr = m_the3DGraphic.SetFunction(&m_theParaboloidFunction);
			ATLASSERT(SUCCEEDED(hr));
			if (FAILED(hr)) {
				return hr;
			}
			break;
		}
	default:
		{
			return E_FAIL;
		}
	}
	
	hr = m_theGraphFrame.Refresh(TRUE);
	ATLASSERT(SUCCEEDED(hr));
	
	return S_OK;
}
