// maindlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINDLG_H__D54DBB15_8C98_41D4_8BCD_A37F98D07B42__INCLUDED_)
#define AFX_MAINDLG_H__D54DBB15_8C98_41D4_8BCD_A37F98D07B42__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Resource.h"

#include "3DGraphFrame.h"
#include "3DGraphic.h"

#include "D3D8App.h"

#include "LightPropsWindow.h"
#include "MaterialPropsWindow.h"
#include "BackColorWindow.h"
#include "FunctionTypeWindow.h"

#include "MathFunctions.h"

class CMainDlg : public CDialogImpl<CMainDlg>,
	public CD3D8Application,
	public CPropertyWindowNotify
{
public:	
/////////////////////////////////////////////////////////////////////////////////////////
// CMainDlg construction/destruction
//

	CMainDlg();

/////////////////////////////////////////////////////////////////////////////////////////
// CMainDlg member variables
//
private:
protected:	
	C3DGraphFrame m_theGraphFrame;	
	C3DGraphic m_the3DGraphic;

// Parameters windows	
	CMaterialPropsWindow m_theMaterialPropsWindow;
	CLightPropsWindow m_theLightPropsWindow;
	CBackColorWindow m_theBackColorWindow;
	CFunctionTypeWindow m_theFunctionTypeWindow;

// Math functions
	CSplashFunction m_theSplashFunction;
	CPlaneFunction	m_thePlaneFunction;
	CParaboloidFunction m_theParaboloidFunction;
public:
/////////////////////////////////////////////////////////////////////////////////////////
// CMainDlg methods 
//
private:
	HRESULT CreatePropertiesWindows();
	HRESULT ShowPropertiesWindow(UINT p_uiPropWindowID, BOOL p_bShow);

	HRESULT MaterialParametersChanged();
	HRESULT LightParametersChanged();
	HRESULT BackColorChanged();
	HRESULT FunctionTypeChanged();
protected:

public:
	BOOL DispatchDialogMessage(LPMSG p_pMsg)
	{
		if (m_hWnd && IsDialogMessage(p_pMsg)) {
			return TRUE;
		}
		if (m_theMaterialPropsWindow.m_hWnd && 
			m_theMaterialPropsWindow.IsDialogMessage(p_pMsg)) {
			return TRUE;
		}
		if (m_theLightPropsWindow.m_hWnd && 
			m_theLightPropsWindow.IsDialogMessage(p_pMsg)) {
			return TRUE;
		}
		if (m_theBackColorWindow.m_hWnd && 
			m_theBackColorWindow.IsDialogMessage(p_pMsg)) {
			return TRUE;
		}
		if (m_theFunctionTypeWindow.m_hWnd && 
			m_theFunctionTypeWindow.IsDialogMessage(p_pMsg)) {
			return TRUE;
		}

		return FALSE;
	}

/////////////////////////////////////////////////////////////////////////////////////////
// CMainDlg's CPropertyWindowNotify implementation
//
public:
	virtual HRESULT HideRequest(UINT p_uiPropWindowID);
	virtual HRESULT ParametersChanged(UINT p_uiPropWindowID);


public:
	
	enum { IDD = IDD_MAINDLG };

	BEGIN_MSG_MAP(CMainDlg)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
		MESSAGE_HANDLER(WM_CLOSE, OnClose)
		MESSAGE_HANDLER(WM_DESTROY, OnDestroyWindow)
		COMMAND_ID_HANDLER(IDC_ABOUT, OnAppAbout)
		COMMAND_ID_HANDLER(IDOK, OnOK)
		COMMAND_ID_HANDLER(IDCANCEL, OnCancel)

		COMMAND_ID_HANDLER(IDC_MATERIALPROPERTIES, OnMenuWindowClick)
		COMMAND_ID_HANDLER(IDC_LIGHTPROPERTIES, OnMenuWindowClick)
		COMMAND_ID_HANDLER(IDC_FUNCTIONTYPE, OnMenuWindowClick)
		COMMAND_ID_HANDLER(IDC_BACKGROUNDCOLOR, OnMenuWindowClick)

	END_MSG_MAP()

	LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		// center the dialog on the screen
		CenterWindow();

		// set icons
		HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
		SetIcon(hIcon, TRUE);
		HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
		SetIcon(hIconSmall, FALSE);

		HRESULT hr;
		

		if (!Direct3DInitOK()) {
			USES_RESSTR;
			::MessageBox(NULL, RESSTR(IDS_ERR_CREATE3DOBJECT), 
				RESSTR(IDS_APPNAME), MB_OK | MB_ICONEXCLAMATION);
			PostMessage(WM_CLOSE);
			return 0;
		}

		hr = m_theGraphFrame.Create3DFrame(GetDlgItem(IDC_FR_GRAPH), &m_the3DGraphic);
		if (FAILED(hr)) {
			USES_RESSTR;
			::MessageBox(NULL, RESSTR(IDS_ERR_CREATE3DFRAME), 
				RESSTR(IDS_APPNAME), MB_OK | MB_ICONEXCLAMATION);
			PostMessage(WM_CLOSE);
			return 0;
		}

		hr = m_the3DGraphic.Create(this, m_theGraphFrame.m_hWnd, 
			&m_thePlaneFunction);
		if (FAILED(hr)) {
			USES_RESSTR;
			::MessageBox(NULL, RESSTR(IDS_ERR_CREATE3DGRAPH), 
				RESSTR(IDS_APPNAME), MB_OK | MB_ICONEXCLAMATION);
			PostMessage(WM_CLOSE);
			return 0;
		}
		
		m_the3DGraphic.ReRender(TRUE);

		hr = CreatePropertiesWindows();
		ATLASSERT(SUCCEEDED(hr));

		hr = ParametersChanged(CMaterialPropsWindow::IDD);
		ATLASSERT(SUCCEEDED(hr));
		hr = ShowPropertiesWindow(CMaterialPropsWindow::IDD, FALSE);
		ATLASSERT(SUCCEEDED(hr));

		hr = ParametersChanged(CLightPropsWindow::IDD);
		ATLASSERT(SUCCEEDED(hr));
		hr = ShowPropertiesWindow(CLightPropsWindow::IDD, FALSE);
		ATLASSERT(SUCCEEDED(hr));

		hr = ParametersChanged(CBackColorWindow::IDD);
		ATLASSERT(SUCCEEDED(hr));
		hr = ShowPropertiesWindow(CBackColorWindow::IDD, FALSE);
		ATLASSERT(SUCCEEDED(hr));
		
		hr = ParametersChanged(CFunctionTypeWindow::IDD);
		ATLASSERT(SUCCEEDED(hr));
		hr = ShowPropertiesWindow(CFunctionTypeWindow::IDD, FALSE);
		ATLASSERT(SUCCEEDED(hr));
		
		
		return TRUE;
	}
	LRESULT OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		// Set bHandled as if we have not processed this message.
		// This is for walking around ATL bug.
		bHandled = FALSE;
		
		DestroyWindow();
		return TRUE;
	}

	LRESULT OnDestroyWindow(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		if (m_theLightPropsWindow.m_hWnd) {
			m_theLightPropsWindow.DestroyWindow();
		}
		
		if (m_theMaterialPropsWindow.m_hWnd) {
			m_theMaterialPropsWindow.DestroyWindow();
		}
		if (m_theBackColorWindow.m_hWnd) {
			m_theBackColorWindow.DestroyWindow();
		}
		if (m_theFunctionTypeWindow.m_hWnd) {
			m_theFunctionTypeWindow.DestroyWindow();
		}
		
		PostQuitMessage(0);
		return 0;
	}
	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CSimpleDialog<IDD_ABOUTBOX, FALSE> dlg;
		dlg.DoModal();
		return 0;
	}

	LRESULT OnOK(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		PostMessage(WM_CLOSE);
		return 0;
	}

	LRESULT OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		PostMessage(WM_CLOSE);
		return 0;
	}

	LRESULT OnMenuWindowClick(WORD /*wNotifyCode*/, WORD wID, 
		HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		HMENU hmnuMain = GetMenu();
		if (!hmnuMain) {
			return 0;
		}
		UINT uiState = GetMenuState(hmnuMain, wID, MF_BYCOMMAND);
		if (-1 == uiState) {
			// No such an item.
			return 0;
		}
		UINT uiWindowID = 0;
		switch (wID)
		{
		case(IDC_MATERIALPROPERTIES):
			{
				uiWindowID = CMaterialPropsWindow::IDD;
				break;
			}
		case(IDC_LIGHTPROPERTIES):
			{
				uiWindowID = CLightPropsWindow::IDD;
				break;
			}
		case(IDC_FUNCTIONTYPE):
			{
				uiWindowID = CFunctionTypeWindow::IDD;
				break;
			}
		case(IDC_BACKGROUNDCOLOR):
			{
				uiWindowID = CBackColorWindow::IDD;
				break;
			}
		default:
			{
				return 0;
			}
		}
		
		HRESULT hr = E_FAIL;
		hr = ShowPropertiesWindow(uiWindowID, 
			(uiState & MF_CHECKED) == 0);
		ATLASSERT(SUCCEEDED(hr));
		
		return 0;
	}
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINDLG_H__D54DBB15_8C98_41D4_8BCD_A37F98D07B42__INCLUDED_)
